/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2005, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * Modifications (C) 2006 Russ Price <kxt@fubegra.net>
 * This is loosely based on Asterisk's app_playback.c
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Play back simulated Panel revertive pulsing sounds based on a four-digit number
 * 
 * \ingroup applications
 */
 
/*
 * $Id: app_rpsim.c,v 1.3 2009/07/14 23:31:34 russ Exp $
 * $Log: app_rpsim.c,v $
 * Revision 1.3  2009/07/14 23:31:34  russ
 * 1.4-compatible version
 *
 * Revision 1.2  2006/08/24 19:00:51  russ
 * Proper build system for compiling separately from Asterisk
 *
 * Revision 1.1  2006/08/24 18:39:29  russ
 * Initial revision
 *
 */

/*** MODULEINFO
    <defaultenabled>no</defaultenabled>
 ***/
#define AST_MODULE "rpsim"

#ifndef ASTERISK_VERSION
	#define ASTERISK_VERSION 4
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <asterisk/utils.h>
#include <asterisk/md5.h>
#include <asterisk/app.h>
#include <asterisk/indications.h>

#if ASTERISK_VERSION < 4
static char *tdesc = "Play back simulated Panel revertive pulsing sounds for a four-digit number";
#endif

static char *app = "RPsim";

static char *synopsis = "Simulate Panel revertive pulsing";

static char *descrip = 
"  RPsim(number|options):  Mimics the revertive pulsing sounds\n"
"that were a characteristic of calling a Bell System Panel switch\n" 
"through a crossbar tandem. See <http://www.widewebs.com/phonetrips/>\n"
"for some examples. This application uses material from the Evan\n" 
"Doorbell tapes.\n"
"The number should be four digits unless you are using MF.\n\n"
"  Options:\n"
"    b_side	- Adds four pulses to second RP digit to simulate\n"
"		  B-side exchange\n"
"    answer	- Answer channel before playing digits.\n"
"    mf         - Play MF tones instead of RP";

static char *mf_subset[] = {
"1300+1500/68,0/68",	/* 0 */
"700+900/68,0/68",	/* 1 */
"700+1100/68,0/68",	/* 2 */
"900+1100/68,0/68",	/* ... */
"700+1300/68,0/68",
"900+1300/68,0/68",
"1100+1300/68,0/68",
"700+1500/68,0/68",
"900+1500/68,0/68",
"1100+1500/68,0/68",	/* 9 */
"1500+1700/68,0/68",	/* ST */
"700+1700/68,0/68",	/* Code 11 */
"900+1700/68,0/68",	/* Code 12 */
"1100+1700/100,0/68",	/* KP1 */
"1300+1700/100,0/68",	/* KP2 */
"480/250,0/68"		/* error */
};

#define ST 10
#define CODE11 11
#define CODE12 12
#define KP1 13
#define KP2 14
#define BOGUS 15


#if ASTERISK_VERSION < 4
STANDARD_LOCAL_USER;
LOCAL_USER_DECL;
#endif

void translate(int number, int b_side, int *data);
int playit(struct ast_channel *chan, char *file);
void play_mf(struct ast_channel *chan, char c);

int playit(struct ast_channel *chan, char *file)
{
	int res;
	res = ast_streamfile(chan, file, chan->language);
	if(!res) {
		res = ast_waitstream(chan, "");
		ast_stopstream(chan);
	} else {
		ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", chan->name, file);
		res = 0;
	}
	return res;
}

void translate(int number, int b_side, int *data)
{
	if(number > 9999) {
		ast_log(LOG_WARNING, "RP simulator number should be four digits; truncating %d to %d\n", number, number % 10000);
		number %= 10000;
	}
	int b = (b_side) ? 4 : 0;
	data[0] = (number / 2000) + 1;
	data[1] = (number % 2000) / 500 + b + 1;
	data[2] = (number % 500) / 100 + 1;
	data[3] = (number % 100) / 10 + 1;
	data[4] = (number % 10) + 1;
}

void play_mf(struct ast_channel *chan, char c)
{
	short int k;
	short int delay = 136;
	switch(c) {
		case '*': k = CODE11; break;
		case '#': k = CODE12; break;
		case 'A': k = KP1; delay = 168; break;
		case 'B': k = KP2; delay = 168; break;
		case 'C': k = ST; break;
		case 'D': k = BOGUS; break;
		default: k = c - 0x30;
	}

	if(k < 0 || k >= sizeof(mf_subset) / sizeof (char *))
		k = BOGUS;

	ast_playtones_start(chan,0,mf_subset[k],0);
	ast_safe_sleep(chan, delay);
	ast_playtones_stop(chan);
}

static int rpsim_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
#if ASTERISK_VERSION < 4
	struct localuser *u;
#else
	struct ast_module_user *u;
#endif
	int option_b_side = 0;
	int option_answer = 0;
	int option_mf = 0;
	int option_kp2 = 0;
	int option_bare = 0;
	char *tmp=NULL;
	int translated[5];
	char fpath[80];
	int i, j, k;

	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(number);
		AST_APP_ARG(options);
	);

	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "RPsim requires an argument (number)\n");
		return -1;
	}

#if ASTERISK_VERSION < 4
	LOCAL_USER_ADD(u);
#else
	u = ast_module_user_add(chan);
#endif

	tmp = ast_strdupa(data);
	if (!tmp) {
		ast_log(LOG_ERROR, "Out of memory!\n");
#if ASTERISK_VERSION < 4
		LOCAL_USER_REMOVE(u);
#else
		ast_module_user_remove(u);
#endif
		return -1;	
	}

	AST_STANDARD_APP_ARGS(args, tmp);

	if (args.options) {
		if (strcasestr(args.options, "b_side"))
			option_b_side = 1;
		if (strcasestr(args.options, "answer"))
			option_answer = 1;
		if (strcasestr(args.options, "mf"))
			option_mf = 1;
		if (strcasestr(args.options, "kp2"))
			option_kp2 = 1;
		if (strcasestr(args.options, "bare"))
			option_bare = 1;
	}
	
	ast_stopstream(chan);
	if(chan->_state != AST_STATE_UP) {
		if(option_answer)
			res = ast_answer(chan);
		if(!res)
			res = ast_safe_sleep(chan, 500);
	}

	if(option_mf) {
		j = strlen(tmp);
		if(!option_bare)
			play_mf(chan, option_kp2 ? 'B' : 'A');

		for(i=0; i<j; i++) {
			play_mf(chan, tmp[i]);
		}

		if(!option_bare)
			play_mf(chan, 'C');
	} else {
		translate(atoi(tmp), option_b_side, translated);
		res = playit(chan, "rpsim/begin");
		for(i=0; i<5; i++) {
			sprintf(fpath, "rpsim/start%d", i);
			res = playit(chan, fpath);

			sprintf(fpath, "rpsim/first%d", i);
			if(translated[i] > 1)
				res = playit(chan, fpath);

			sprintf(fpath, "rpsim/middle%d", i);
			k = translated[i] - 2;
			for(j=0; j<k; j++)
				res = playit(chan, fpath);

			sprintf(fpath, "rpsim/last%d", i);
			res = playit(chan, fpath);
		}
		res = playit(chan, "rpsim/end");
	}
#if ASTERISK_VERSION < 4
	LOCAL_USER_REMOVE(u);
#else
	ast_module_user_remove(u);
#endif
	return res;
}

#if ASTERISK_VERSION >= 4
static int unload_module(void)
{
	int res;
	res = ast_unregister_application(app);
	return res;
}

static int load_module(void)
{
	return ast_register_application(app, rpsim_exec, synopsis, descrip);
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "RPSIM Revertive Pulsing Simulator");

#else
int unload_module(void)
{
	int res;

	res = ast_unregister_application(app);
	
	STANDARD_HANGUP_LOCALUSERS;

	return res;	
}

int load_module(void)
{
	return ast_register_application(app, rpsim_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
#endif
